package org.example;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import javax.swing.JSlider;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;


import java.util.Map;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;


import static org.example.Modes.*;


public class Main extends JFrame {
    private ArrayList<Primitive> primitives = new ArrayList<>();
    private JSlider scaleSlider;

    private Modes mode = null;

    private boolean isDragging = false;
    private int dragStartX, dragStartY;

    private Primitive currentPrimitive = null;

    private JLabel convexityLabel;


    private int alpha;

    private Polygon polygon;

    public void addTreeFromFile(String fileName, int maxSteps){
        try {
            BufferedReader reader = new BufferedReader(new FileReader(fileName));
            String[] firstLine = reader.readLine().split(" ");
            double angle = Double.parseDouble(firstLine[0]);
            double startDirection = Double.parseDouble(firstLine[1]);
            String atom = reader.readLine();
            Map<Character, String> rules = new HashMap<>();
            String line;
            while ((line = reader.readLine()) != null) {
                String[] parts = line.split("->");
                rules.put(parts[0].charAt(0), parts[1]);
            }
            StringBuilder axiom = new StringBuilder();
            for (int i = 0; i < maxSteps; i++) {
                axiom = new StringBuilder();
                for (Character character : atom.toCharArray()) {
                    if (rules.containsKey(character)) {
                        axiom.append(rules.get(character));
                    } else {
                        axiom.append(character);
                    }
                }
                atom = axiom.toString();
            }
            Fractal fractal = new Tree(400, 570, atom, startDirection, angle);
            currentPrimitive = fractal;
            double height = fractal.maxY - fractal.minY;
            double width = fractal.maxX - fractal.minX;
            double koef = 1;
            if(width <= height){
                koef = (getHeight()-200)/height;
            }else{
                koef = getWidth()/width;
            }
            System.out.println(koef);
            fractal.scale(1.7,1.7);
            primitives.add(fractal);
        } catch (FileNotFoundException e) {
            throw new RuntimeException(e);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }

    }

    public void addFractalFromFile(String fileName, int maxSteps){
        try {
            BufferedReader reader = new BufferedReader(new FileReader(fileName));
            String[] firstLine = reader.readLine().split(" ");
            double angle = Double.parseDouble(firstLine[0]);
            double startDirection = Double.parseDouble(firstLine[1]);
            String atom = reader.readLine();
            Map<Character, String> rules = new HashMap<>();
            String line;
            while ((line = reader.readLine()) != null) {
                String[] parts = line.split("->");
                rules.put(parts[0].charAt(0), parts[1]);
            }
            StringBuilder axiom = new StringBuilder();
            for (int i = 0; i < maxSteps; i++) {
                axiom = new StringBuilder();
                for (Character character : atom.toCharArray()) {
                    if (rules.containsKey(character)) {
                        axiom.append(rules.get(character));
                    } else {
                        axiom.append(character);
                    }
                }
                atom = axiom.toString();
            }
            Fractal fractal = new Fractal(400, 150, atom, startDirection, angle);
            currentPrimitive = fractal;
            double height = fractal.maxY - fractal.minY;
            double width = fractal.maxX - fractal.minX;
            double koef = 1;
            if(width <= height){
                koef = (getHeight()-200)/height;
            }else{
                koef = (getWidth()-200)/width;
            }
            System.out.println(koef);
            fractal.scale(koef,koef);
            primitives.add(fractal);
        } catch (FileNotFoundException e) {
            throw new RuntimeException(e);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }

    }
    public Main() {
        setTitle("App");
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setSize(800, 800);

        addFractalFromFile("gosper.txt", 5);
        //addTreeFromFile("tree.txt", 12);

        addMouseListener(new MouseAdapter() {
            @Override
            public void mousePressed(MouseEvent e) {
                // Проверяем, попали ли мы в фигуру
                for (Primitive primitive : primitives) {
                    if (isPointInsidePolygon(new Point(e.getX(), e.getY()), primitive)) {
                        isDragging = true;
                        dragStartX = e.getX();
                        dragStartY = e.getY();
                        break;
                    }
                }
            }

            @Override
            public void mouseReleased(MouseEvent e) {
                isDragging = false;
            }
        });

        addMouseMotionListener(new MouseAdapter() {
            @Override
            public void mouseDragged(MouseEvent e) {
                if (isDragging && currentPrimitive != null) {
                    int deltaX = e.getX() - dragStartX;
                    int deltaY = e.getY() - dragStartY;
                    currentPrimitive.translate(deltaX, deltaY);
                    dragStartX = e.getX();
                    dragStartY = e.getY();
                    repaint(); // Добавьте этот вызов для обновления отрисовки в реальном времени
                }
            }
        });



        buildPanel();
        setVisible(true);
        convexityLabel = new JLabel("");
        add(convexityLabel, BorderLayout.SOUTH);
    }


    public void buildPanel() {
        JPanel buttonPanel = new JPanel();

        // Создаем ползунок для масштабирования
        scaleSlider = new JSlider(JSlider.HORIZONTAL, 1, (int) (currentPrimitive.scaleX*100)*5, ((int) (currentPrimitive.scaleX*100)));
        //scaleSlider.setMajorTickSpacing(500);
        //scaleSlider.setMinorTickSpacing(500);
        scaleSlider.setPreferredSize(new Dimension(getWidth(), 40)); // Например, ширина экрана, высота 40
        scaleSlider.setPaintTicks(true);
        scaleSlider.setPaintLabels(true);
        scaleSlider.addChangeListener(e -> {
            int scaleFactor = scaleSlider.getValue();
            if (currentPrimitive != null) {
                // Масштабируем текущую фигуру
                double scaleX = scaleFactor / 100.0;
                double scaleY = scaleFactor / 100.0;
                System.out.println(scaleX);
                currentPrimitive.scale(scaleX, scaleY);
                repaint();
            }
        });

        buttonPanel.add(scaleSlider);

        add(buttonPanel, BorderLayout.NORTH);
    }


    @Override
    public void paint(Graphics g) {
        super.paint(g);
        for (int i = 0; i < primitives.size(); i++) {
            try {
                primitives.get(i).draw(g);
            } catch (InterruptedException e) {
                throw new RuntimeException(e);
            }
        }
    }

    public static void main(String[] args) {
        SwingUtilities.invokeLater(Main::new);
    }


    private boolean isPointInsidePolygon(Point point, Primitive polygon) {
        int n = polygon.points.size();
        if (n < 3) {
            return false;
        }

        double minX = Integer.MAX_VALUE;
        double maxX = Integer.MIN_VALUE;
        double minY = Integer.MAX_VALUE;
        double maxY = Integer.MIN_VALUE;

        for (Point vertex : polygon.points) {
            if (vertex.x < minX) {
                minX = vertex.x;
            }
            if (vertex.x > maxX) {
                maxX = vertex.x;
            }
            if (vertex.y < minY) {
                minY = vertex.y;
            }
            if (vertex.y > maxY) {
                maxY = vertex.y;
            }
        }

        if (point.x < minX || point.x > maxX || point.y < minY || point.y > maxY) {
            return false;
        }

        boolean isInside = false;
        int j = n - 1;
        for (int i = 0; i < n; i++) {
            boolean isVertexAbove = polygon.points.get(i).y > point.y;
            boolean isPrevVertexAbove = polygon.points.get(j).y > point.y;

            boolean isIntersecting = point.x < (polygon.points.get(j).x - polygon.points.get(i).x) * (point.y - polygon.points.get(i).y) /
                    (polygon.points.get(j).y - polygon.points.get(i).y) + polygon.points.get(i).x;

            if ((isVertexAbove != isPrevVertexAbove) && isIntersecting) {
                isInside = !isInside;
            }

            j = i;
        }

        return isInside;
    }



}
