package org.example;

import java.awt.*;
import java.awt.geom.Line2D;
import java.util.ArrayList;
import java.util.Random;
import java.util.Stack;
import java.util.Timer;

class Point {
    double x;
    double y;

    Point(double x, double y){
        this.x = x;
        this.y = y;
    }
}

class Primitive {
    boolean filled = false;
    public double scaleX = 1;
    public ArrayList<Point> points;
    public Point center;
    public Color color = Color.BLACK;

    public Primitive(int x, int y) {
        points = new  ArrayList<Point>();
        points.add(new Point(x,y));
    }

    public void setColor(Color color){
        this.color = color;
    }
    public Primitive(Point point) {
        points = new  ArrayList<Point>();
        points.add(point);
    }

    public void translate(double dx, double dy) {
        double[][] translationMatrix = {
                {1, 0, dx},
                {0, 1, dy},
                {0, 0, 1}
        };

        for (Point point1 : points) {
            transformPoint(translationMatrix, point1);
        }
        transformPoint(translationMatrix, center);
    }


    public void scale(double koeffX, double koeffY) {
        double[][] rotationMatrix = {
                {(double) 1 /koeffX, 0, 0},
                {0, (double) 1 /koeffY, 0},
                {0, 0, 1}
        };
        if(this.center == null){
            calculateCenter();
        }
        double x = center.x;
        double y = center.y;
        translate(-center.x, -center.y);
        for (Point point : points) {
            transformPoint(rotationMatrix, point);
        }
        translate(x, y);
        calculateCenter();
    }



    public void calculateCenter() {
        ArrayList<Point> points = this.points;
        int totalX = 0;
        int totalY = 0;
        for (Point point : points) {
            totalX += point.x;
            totalY += point.y;
        }
        int centerX = totalX / points.size();
        int centerY = totalY / points.size();

        this.center = new Point(centerX, centerY);
    }

    public void transformPoint(double[][] translationMatrix, Point point){
        if(point == null){
            return;
        }
        double x = point.x;
        double y = point.y;
        double newX = translationMatrix[0][0] * x + translationMatrix[0][1] * y + translationMatrix[0][2];
        double newY = translationMatrix[1][0] * x + translationMatrix[1][1] * y + translationMatrix[1][2];
        point.x = (int) newX;
        point.y = (int) newY;
    }

    public void draw(Graphics g) throws InterruptedException {
        g.setColor(color);
        int radius = 5 / 2;
        //g.fillOval(points.get(0).x - radius, points.get(0).y - radius, 5, 5);
    }

}



class FractalState {
    private double x;
    private double y;
    private double direction;

    private float lineWidth; // Толщина линий
    private Color currentColor; // Текущий цвет

    private float lineLength; // Толщина линий
    private int level;
    public FractalState(double x, double y, double direction, float lineWidth, Color currentColor, float lineLength, int level) {
        this.x = x;
        this.y = y;
        this.direction = direction;
        this.lineWidth = lineWidth;
        this.currentColor = currentColor;
        this.lineLength = lineLength;
        this.level = level;
    }

    public FractalState(double x, double y, double direction) {
        this.x = x;
        this.y = y;
        this.direction = direction;
    }

    public double getX() {
        return x;
    }

    public double getY() {
        return y;
    }

    public double getDirection() {
        return direction;
    }

    public float getLineWidth() {
        return lineWidth;
    }

    public Color getCurrentColor() {
        return currentColor;
    }

    public float getLineLength() {
        return lineLength;
    }

    public int getLevel() {
        return level;
    }
}


class Fractal extends Primitive{

    public double minY;
    public double maxX;
    public double minX;
    public double maxY;

    protected String axiom;
    protected double angle;
    protected Stack<FractalState> stateStack;

    protected double scaleY = 1;

    protected double direction;

    public Fractal(int x, int y) {
        super(x, y);
    }
    public Fractal(int x, int y, String axiom, double direction, double angle) {
        super(x, y);
        minY = Double.MAX_VALUE;
        minX = Double.MAX_VALUE;
        maxY = Double.MIN_VALUE;
        maxX = Double.MIN_VALUE;
        scaleY = 1;
        scaleX = 1;
        this.axiom = axiom;
        this.direction = direction;
        this.angle = angle;
        this.stateStack = new Stack<FractalState>();
        this.calculate();
    }

    public void scale(double koeffX, double koeffY) {
        Point buf = points.get(0);
        points.clear();
        points.add(buf);
        scaleX = koeffX;
        scaleY = koeffY;
        calculate();
        System.out.println(koeffX);
    }

    public void calculate(){
        int i = 0;

        double startX = this.points.get(0).x;
        double startY = this.points.get(0).y;
        double x = startX;
        double y = startY;
        double tempDirection = direction;
        Random random = new Random();

        for (char r : axiom.toCharArray()) {
            if (r == 'F' ) {
                double newX = x + Math.cos(Math.toRadians(tempDirection)) * scaleX;
                double newY = y + Math.sin(Math.toRadians(tempDirection)) * scaleY;
                points.add(new Point(newX, newY));
                if(newY > maxY){
                    maxY = newY;
                }
                if(newY < minY){
                    minY = newY;
                }
                if(newX > maxX){
                    maxX = newX;
                }
                if(newX < minX){
                    minX = newX;
                }
                x = newX;
                y = newY;
            } else if (r == '+') {
                tempDirection += angle;
            } else if (r == '-') {
                tempDirection -= angle;
            } else if (r == '[') {
                stateStack.push(new FractalState(x, y, tempDirection));
            } else if (r == ']') {
                FractalState state = stateStack.pop();
                x = (int) state.getX();
                y = (int) state.getY();
                tempDirection = state.getDirection();
            }
        }

    }

    public void draw(Graphics g) throws InterruptedException {
        Graphics2D g2d = (Graphics2D) g;
        calculateCenter();

        Point point1 = points.get(0);
        g.setColor(Color.RED);
        g2d.setStroke(new BasicStroke(5));
        int radius = 10; // Радиус точки
        g2d.fillOval((int)point1.x - radius, (int)point1.y - radius, 2 * radius, 2 * radius);
        g.setColor(color);
        g2d.setStroke(new BasicStroke(1));

        Point point2 = null;
        for(int j = 0; j < points.size()-1; j++){
            point1 = points.get(j);
            point2 = points.get(j+1);
            g2d.draw(new Line2D.Double(point1.x, point1.y, point2.x, point2.y));
            //Thread.sleep(1000);
        }

    }
}

class Line{
    Line2D line;
    private float lineWidth; // Толщина линий
    private Color currentColor; // Текущий цвет

    private int level;
    public Line(Line2D line){
        this.line = line;
        this.lineWidth = 1;
        this.currentColor = Color.black;
    }

    void changeLine(double koeff){
        line.setLine(new Line2D.Double(line.getX1()*koeff, line.getY1()*koeff, line.getX2()*koeff, line.getY2()*koeff));
    }

    public Line(Line2D line, float lineWidth){
        this.line = line;
        this.lineWidth = lineWidth;
        this.currentColor = Color.black;
    }

    public int getLevel() {
        return level;
    }

    public void setLevel(int level) {
        this.level = level;
    }

    public Line2D getLine() {
        return line;
    }

    public float getLineWidth() {
        return lineWidth;
    }

    public Color getCurrentColor() {
        return currentColor;
    }

    public void setLineWidth(float lineWidth) {
        this.lineWidth = lineWidth;
    }

    public void setCurrentColor(Color currentColor) {
        this.currentColor = currentColor;
    }
}

class Tree extends Fractal{
    private ArrayList<Line> lines;
    private float lineWidth; // Толщина линий
    private Color currentColor; // Текущий цвет

    private int level;
    private float lineLength;
    public Tree(int x, int y, String axiom, double direction, double angle) {
        super(x, y, axiom, direction, angle);
        lineWidth = 10.0f;
        lineLength = 50;
        currentColor = new Color(139, 69, 19);
    }

    public void calculate(){
        lines = new ArrayList<>();
        int i = 0;

        double startX = this.points.get(0).x;
        double startY = this.points.get(0).y;
        double x = startX;
        double y = startY;
        double tempDirection = direction;
        Random random = new Random();

        for (char r : axiom.toCharArray()) {
            if (r == 'F' || r == 'X') {
                double newX = x + Math.cos(Math.toRadians(tempDirection)) * lineLength * scaleX;
                double newY = y + Math.sin(Math.toRadians(tempDirection)) * lineLength* scaleX;
                Line line = new Line(new Line2D.Double(x, y, newX, newY));
                line.setLineWidth(lineWidth);
                line.setCurrentColor(currentColor);
                line.setLevel(level);
                lines.add(line);
                points.add(new Point(newX, newY));
                if(newY > maxY){
                    maxY = newY;
                }
                if(newY < minY){
                    minY = newY;
                }
                if(newX > maxX){
                    maxX = newX;
                }
                if(newX < minX){
                    minX = newX;
                }
                x = newX;
                y = newY;
                lineWidth *= 0.7f;
                lineLength *= 0.8f;
                level = level+1;
            } else if (r == '+') {
                tempDirection +=  random.nextInt(46);
            } else if (r == '-') {
                tempDirection -=  random.nextInt(46);
            } else if (r == '[') {
                stateStack.push(new FractalState(x, y, tempDirection, lineWidth, Color.black, lineLength, level));
            } else if (r == ']') {
                FractalState state = stateStack.pop();
                x = (int) state.getX();
                y = (int) state.getY();
                lineWidth = state.getLineWidth();
                lineLength = state.getLineLength();
                level = state.getLevel();
                tempDirection = state.getDirection();
            }
        }

    }

    public void scale(double koeffX, double koeffY) {
        Point buf = points.get(0);
        lines.clear();
        points.clear();
        points.add(buf);
        scaleX = koeffX;
        scaleY = koeffY;
        lineWidth = 10.0f;
        lineLength = 50;
        level = 0;
        calculate();
        System.out.println(koeffX);
    }

    public void transformLine(double[][] translationMatrix, Line2D line){
        if(line == null){
            return;
        }
        double x = line.getX1();
        double y = line.getY1();
        double newX = translationMatrix[0][0] * x + translationMatrix[0][1] * y + translationMatrix[0][2];
        double newY = translationMatrix[1][0] * x + translationMatrix[1][1] * y + translationMatrix[1][2];
        double x2 = line.getX1();
        double y2 = line.getY1();
        double newX2 = translationMatrix[0][0] * x2 + translationMatrix[0][1] * y2 + translationMatrix[0][2];
        double newY2 = translationMatrix[1][0] * x2 + translationMatrix[1][1] * y2 + translationMatrix[1][2];
        line.setLine(new Line2D.Double(newX, newY, newX2, newY2));
    }


    public void translate(double dx, double dy) {
        double[][] translationMatrix = {
                {1, 0, dx},
                {0, 1, dy},
                {0, 0, 1}
        };

        for (Line line : lines) {
            transformLine(translationMatrix, line.line);
        }
    }

    public void draw(Graphics g) throws InterruptedException {
        Graphics2D g2d = (Graphics2D) g;
        calculateCenter();

        Point point1 = points.get(0);
        g.setColor(Color.RED);
        g2d.setStroke(new BasicStroke(5));
        int radius = 10; // Радиус точки
        g2d.fillOval((int)point1.x - radius, (int)point1.y - radius, 2 * radius, 2 * radius);
        g.setColor(color);
        g2d.setStroke(new BasicStroke(1));
        Color startColor = new Color(94, 52, 22); // Коричневый цвет
        Color endColor = new Color(1, 157, 1);
        for (Line line : lines) {

            float ratio = (float) (line.getLevel()) / lines.get(lines.size()-1).getLevel();
            int red = (int) (startColor.getRed() * (1 - ratio) + endColor.getRed() * ratio);
            int green = (int) (startColor.getGreen() * (1 - ratio) + endColor.getGreen() * ratio);
            int blue = (int) (startColor.getBlue() * (1 - ratio) + endColor.getBlue() * ratio);

            Color currentColor = new Color(red, green, blue);
            g2d.setStroke(new BasicStroke(line.getLineWidth()));
            g.setColor(currentColor);
            g2d.draw(line.line);
            g2d.setStroke(new BasicStroke(1));
        }

    }

}
