package org.example;

import javax.swing.*;
import java.awt.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import static java.lang.Math.abs;

class Point{
    Point2D.Double point;
    boolean isFictive;

    public Point(Point2D.Double point, boolean isFictive) {
        this.point = point;
        this.isFictive = isFictive;
    }

    public Point(double x, double y){
        this.point = new Point2D.Double(x,y);
        this.isFictive = false;
    }
    public Point(double x, double y, boolean isFictive){
        this.point = new Point2D.Double(x,y);
        this.isFictive = isFictive;
    }

    public double getX(){
        return this.point.x;
    }

    public double getY(){
        return this.point.y;
    }
}

public class Main extends JFrame {
    private List<Point> controlPoints = new ArrayList<>();
    private List<List<Point>> bezierCurves = new ArrayList<>();

    private Point selectedPoint = null;
    private boolean isDragging = false;
    public void addPoint(Point point){
        controlPoints.add(point);
        if (controlPoints.size() >= 4) {
            if (controlPoints.size() > 4) {
                addFictive(controlPoints.size()-1);
            }
            updateBezierCurves();
        }

    }

    public void deleteOnePoint(Point closestPoint){
        if(controlPoints.size() > 4) {
            List<Point> list = controlPoints.stream()
                    .filter(point -> !point.isFictive)
                    .filter(point -> point != closestPoint)
                    .toList();
            controlPoints.clear();
            for (Point point : list) {
                addPoint(point);
            }
            updateBezierCurves();
        }
    }

    public Main() {
        setTitle("Composite Cubic Bezier Curve Visualization");
        setSize(800, 600);
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setVisible(true);

        MyCanvas canvas = new MyCanvas();
        add(canvas);
        JButton clearButton = new JButton("Очистить экран");
        clearButton.addActionListener(e -> {
            controlPoints.clear();
            bezierCurves.clear();
            canvas.repaint();
        });

        // Добавьте кнопку в панель
        JPanel buttonPanel = new JPanel();
        buttonPanel.add(clearButton);

        JButton changeMode = new JButton("Показать фиктивные точки");
        changeMode.addActionListener(e -> {
            canvas.isPaintingFictive = !canvas.isPaintingFictive;
            canvas.repaint();
        });

        buttonPanel.add(changeMode);
        add(buttonPanel, BorderLayout.SOUTH);

        canvas.addMouseListener(new MouseAdapter() {
            @Override
            public void mousePressed(MouseEvent e) {
                double mouseX = e.getX();
                double mouseY = e.getY();

                if (SwingUtilities.isRightMouseButton(e)) {
                    if (!controlPoints.isEmpty()) {
                        Point closestPoint = findClosestPoint(mouseX, mouseY, 10);
                        if (closestPoint != null) {
                            deleteOnePoint(closestPoint);
                            canvas.repaint();
                        }
                    }
                } else {
                    selectedPoint = findClosestPoint(mouseX, mouseY, 15);
                    if (selectedPoint != null ) {
                        isDragging = true;
                    }else {
                        addPoint(new Point(mouseX, mouseY));
                        canvas.repaint();
                    }
                }
            }
        });



        canvas.addMouseMotionListener(new MouseMotionAdapter() {
            @Override
            public void mouseDragged(MouseEvent e) {
                if (isDragging && selectedPoint != null) {
                    double mouseX = e.getX();
                    double mouseY = e.getY();

                    selectedPoint.point.setLocation(mouseX, mouseY);
                    List<Point> list = controlPoints.stream()
                            .filter(point -> !point.isFictive)
                            .toList();
                    controlPoints.clear();
                    for (Point point : list){
                        addPoint(point);
                    }

                    updateBezierCurves();
                    canvas.repaint();
                }
            }
        });

        canvas.addMouseListener(new MouseAdapter() {
            @Override
            public void mouseReleased(MouseEvent e) {
                if (isDragging) {
                    isDragging = false;
                }
            }
        });
    }

    public void addFictive(int ind){
        Point p1 = controlPoints.get(ind - 2);
        Point p2 = controlPoints.get(ind -1);
        double t = 0.5;
        double newX = p1.getX() + (p2.getX() - p1.getX()) * t;
        double newY = p1.getY() + (p2.getY() - p1.getY()) * t;
        Point p3 = controlPoints.get(ind -1);
        Point p4 = controlPoints.get(ind );
        double t2 = 0.5;
        double newX2 = p3.getX() + (p4.getX() - p3.getX()) * t2;
        double newY2 = p3.getY() + (p4.getY() - p3.getY()) * t2;
        if(p1.isFictive) {
            controlPoints.remove(p1);
        }else{
            controlPoints.add(ind - 1, new Point(newX, newY, true));
            controlPoints.add(ind +1, new Point(newX2, newY2, true));
        }
    }
    private Point findClosestPoint(double mouseX, double mouseY, int radius) {
        Point closestPoint = null;
        double minDistance = Double.MAX_VALUE;

        for (Point point : controlPoints) {
            if (!point.isFictive) {
                double distance = Point2D.distance(mouseX, mouseY, point.getX(), point.getY());
                if (distance < minDistance) {
                    minDistance = distance;
                    closestPoint = point;
                }
            }
        }
        if(closestPoint == null){
            return null;
        }

        return abs(closestPoint.getX() - mouseX) < radius && abs(closestPoint.getY() - mouseY) < radius ? closestPoint : null;
    }


    private class MyCanvas extends JPanel {

        public boolean isPaintingFictive = false;
        @Override
        protected void paintComponent(Graphics g) {
            super.paintComponent(g);
            Graphics2D g2d = (Graphics2D) g;

            g2d.setColor(Color.RED);
            for (Point point : controlPoints) {
                if(point.isFictive){
                    if(isPaintingFictive) {
                        g2d.setColor(Color.BLACK);
                    }else {
                        continue;
                    }
                }
                int x = (int) point.getX();
                int y = (int) point.getY();
                g2d.fillOval(x - 5, y - 5, 10, 10);
                g2d.setColor(Color.RED);
            }

            g2d.setColor(Color.BLUE);
            g2d.setStroke(new BasicStroke(2));
            if(bezierCurves.size() == 2){
                int e = 3;
            }

            for (List<Point> curve : bezierCurves) {
                if (curve.size() >= 4) {
                    int numSegments = 500;
                    double[] xPoints = new double[numSegments + 1];
                    double[] yPoints = new double[numSegments + 1];
                    for (int i = 0; i <= numSegments; i++) {
                        double t = (double) i / numSegments;
                        Point point = blendFunction(curve.get(0), curve.get(1), curve.get(2), curve.get(3), t);
                        xPoints[i] = point.getX();
                        yPoints[i] = point.getY();
                    }
                    for (int i = 1; i <= numSegments; i++) {
                        int x1 = (int) xPoints[i - 1];
                        int y1 = (int) yPoints[i - 1];
                        int x2 = (int) xPoints[i];
                        int y2 = (int) yPoints[i];
                        g2d.drawLine(x1, y1, x2, y2);
                    }
                }
            }
        }

        private Point blendFunction(Point p0, Point p1, Point p2, Point p3, double t) {
            double u = 1 - t;
            double tt = t * t;
            double uu = u * u;
            double uuu = uu * u;
            double ttt = tt * t;

            double x = p0.getX() * uuu + 3 * p1.getX() * t * uu + 3 * p2.getX() * tt * u + p3.getX() * ttt;
            double y = p0.getY() * uuu + 3 * p1.getY() * t * uu + 3 * p2.getY() * tt * u + p3.getY() * ttt;

            return new Point(x,y);
        }

        private double factorial(int n) {
            if (n == 0) return 1;
            double result = 1;
            for (int i = 1; i <= n; i++) {
                result *= i;
            }
            return result;
        }


    }


    private void updateBezierCurves() {
        bezierCurves.clear();
        int numPoints = controlPoints.size();
        for (int i = 0; i < numPoints; i += 3) {
            if (i + 3 <= numPoints) {
                bezierCurves.add(controlPoints.subList(i, i + 4));
            }
        }
    }

    public static void main(String[] args) {
        SwingUtilities.invokeLater(() -> {
            new Main();
        });
    }
}
